///File : DataManager.cpp
//------------------------------------------------------------------------------
/*
    This file is part of LCDnurse.

    LCDnurse is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    LCDnurse is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with LCDnurse.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "DataManager.hpp"
//------------------------------------------------------------------------------
DataManager::DataManager()
{
    FindStandardPaths();
    m_backgroundColor = BLACK_CODE;
    m_fileChanged = false;
}
//------------------------------------------------------------------------------
void DataManager::FindStandardPaths()
{
    // User config dir
    int result = 0;
    std::string userDir = getenv("HOME");
    std::string userConfigDir = userDir + std::string("/.lcdnursesaver");
    m_filename = userConfigDir + std::string("/lcdnursesaver.conf");

    std::string command = std::string("/usr/bin/test ! -d ") + userConfigDir +  std::string(" && /bin/mkdir ") + userConfigDir;
    result = system(command.c_str());
}
//------------------------------------------------------------------------------
bool DataManager::LoadCfg()
{
    std::ifstream file(m_filename.c_str(), std::ios::in);  // on ouvre en lecture

    if(file)  // si l'ouverture a fonctionné
    {
        std::string str;

        std::getline(file, str);
        int period = stringToInt(str);
        m_frequence = (1.0f / float(period)) * 200000.0f;

        std::getline(file, str);
        int rectCount = stringToInt(str);

        for(int i = 0; i < rectCount; i++)
        {
            RectStr tmpRect;
            std::getline(file, str);
            tmpRect.rect.SetX(stringToInt(str));

            std::getline(file, str);
            tmpRect.rect.SetY(stringToInt(str));

            std::getline(file, str);
            tmpRect.rect.SetWidth(stringToInt(str));

            std::getline(file, str);
            tmpRect.rect.SetHeight(stringToInt(str));

            std::getline(file, str);
            char colorLetter = str[0];

            switch(colorLetter)
            {
                case 'R':
                    tmpRect.color = RED_CODE;
                    break;
                case 'G':
                    tmpRect.color = GREEN_CODE;
                    break;
                case 'B':
                    tmpRect.color = BLUE_CODE;
                    break;
                case 'W':
                    tmpRect.color = WHITE_CODE;
                    break;
                default :
                    tmpRect.color = WHITE_CODE;
                    break;
            }

            m_Rects.push_back(tmpRect);
        }

        file.close();

        m_fileChanged = false;
    }
    else
    {
        m_fileChanged = true;
        m_frequence = 200;

        std::cerr << "Can't open config file \"" << m_filename << "\"\n\\==> Load default conf." << std::endl;
        return 0;
    }

    return 1;
}
//------------------------------------------------------------------------------

bool DataManager::SaveCfg()
{
    std::ofstream file(m_filename.c_str(), std::ios::out | std::ios::trunc);  // on ouvre en ecriture

    if(file)
    {
        int period = (1.0f / float(m_frequence)) * 250000.0f;
        file << period << std::endl;

        file << GetRectCount() << std::endl;

        for(int i = 0; i < GetRectCount(); i++)
        {
            file << m_Rects[i].rect.GetX() << std::endl;
            file << m_Rects[i].rect.GetY() << std::endl;
            file << m_Rects[i].rect.GetWidth() << std::endl;
            file << m_Rects[i].rect.GetHeight() << std::endl;

            char colorLetter = 'W';
            switch(m_Rects[i].color)
            {
                case RED_CODE:
                    colorLetter = 'R';
                    break;
                case GREEN_CODE:
                    colorLetter = 'G';
                    break;
                case BLUE_CODE:
                    colorLetter = 'B';
                    break;
                case WHITE_CODE:
                    colorLetter = 'W';
                    break;
            }
            file << colorLetter << std::endl;
        }

        file.close();
    }
    else
    {
        return 0;
        std::cerr << "Can't save config \"" << m_filename << "\"" << std::endl;
    }

    return 1;
}
//------------------------------------------------------------------------------

std::vector<RectStr> DataManager::GetRects() const
{
    return m_Rects;
}
//------------------------------------------------------------------------------

bool DataManager::DeleteRect(const wxPoint &pos, wxRect &deletedRect)
{
    for(int i = m_Rects.size() - 1; i >= 0; i--) // On commence par la fin pour que le rect le plus visible soit supprimé.
    {
        if(m_Rects[i].rect.Contains(pos))
        {
            deletedRect = m_Rects[i].rect;
            m_Rects.erase(m_Rects.begin() + i);
            m_fileChanged = true;
            return true;
        }
    }
    return false;
}
//------------------------------------------------------------------------------

void DataManager::ClearRects()
{
    if(m_Rects.size() > 0)
    {
        m_Rects.clear();
        m_fileChanged = true;
    }
}
//------------------------------------------------------------------------------

void DataManager::AddRect(const wxRect &rect, const char &color)
{
    if(color >= 0 && color <= COLORS_LIST_NO_BLACK_SIZE - 1)
    {
        RectStr tmpRect;
        tmpRect.color = color;
        tmpRect.rect = rect;

        m_Rects.push_back(tmpRect);

        m_fileChanged = true;
    }
}
//------------------------------------------------------------------------------

size_t DataManager::GetRectCount() const
{
    return m_Rects.size();
}
//------------------------------------------------------------------------------


char DataManager::GetBackgroundColor() const
{
    return m_backgroundColor;
}
//------------------------------------------------------------------------------

void DataManager::SetBackgroundColor(const char &color)
{
    if(color >= 0 && color <= COLORS_LIST_SIZE - 1)
    {
        m_backgroundColor = color;
    }
}
//------------------------------------------------------------------------------
void DataManager::NextBackgroundColor()
{
    if(GetBackgroundColor() == COLORS_LIST_SIZE - 1)
    {
        SetBackgroundColor(0);
    }
    else
    {
        SetBackgroundColor(GetBackgroundColor() + 1);
    }
}
//------------------------------------------------------------------------------
void DataManager::PreviousBackgroundColor()
{
    if(GetBackgroundColor() == 0)
    {
        SetBackgroundColor(COLORS_LIST_SIZE - 1);
    }
    else
    {
        SetBackgroundColor(GetBackgroundColor() - 1);
    }
}
//------------------------------------------------------------------------------
int DataManager::GetFrequence() const
{
    return m_frequence;
}
//------------------------------------------------------------------------------

void DataManager::SetFrequence(const int &frequence)
{
    m_frequence = frequence;
    m_fileChanged = true;
}
//------------------------------------------------------------------------------
int DataManager::GetPeriod() const
{
    return (1.f / float(m_frequence)) * 1000.f;
}
//------------------------------------------------------------------------------

void DataManager::SetPeriod(const int &period)
{
    m_frequence = (1.f / float(period)) * 1000.f;
    m_fileChanged = true;
}
//------------------------------------------------------------------------------

bool DataManager::IsFileChanged()
{
    return m_fileChanged;
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
std::string intToString(int nb)
{
    std::stringstream ss;
    std::string str;
    ss << nb;
    ss >> str;
    return str;
}

//------------------------------------------------------------------------------
int stringToInt(std::string nbStr)
{
    std::stringstream ss;
    int nb;
    ss << nbStr;
    ss >> nb;
    return nb;
}
//------------------------------------------------------------------------------

float stringToFloat(std::string nbStr)
{
    std::stringstream ss;
    double nb;
    ss << nbStr;
    ss >> nb;
    return nb;
}
//------------------------------------------------------------------------------

std::string floatToString(float nb)
{
    std::stringstream ss;
    std::string str;
    ss << nb;
    ss >> str;
    return str;
}

